# Example 2008 MC files for Brunel. For real data use 2008-TED-Data.py
# This file must be given as last argument to gaudirun.py, after Brunel<conf>.py

# Syntax is:
#   gaudirun.py Brunel<conf>.py 2008-Files.py
#
from Gaudi.Configuration import *
from Configurables import Escher, LHCbApp

#-- File catalogs. First one is read-write
FileCatalog().Catalogs = [ "xmlcatalog_file:MyCatalog.xml",
                           "xmlcatalog_file:$BRUNELROOT/job/NewCatalog.xml" ]

#-- Main ('signal') event input
Escher().InputType = "digi"
inputType   = Escher().getProp("InputType").upper()

if inputType == "MDF":
    import sys
    print "**********************************************************************"
    print " 2008 real data requires special options. Use 2008-TED-Data.py instead"
    print "**********************************************************************"
    sys.exit()

elif inputType == "DST":
    # Example POOL DST (reprocessing)
    datasetName =  "00001820_00000001"
    # No 2008 DST file yet, format is as follows:
    # EventSelector().Input = ["DATAFILE='PFN:castor:/castor/cern.ch/grid/lhcb/production/DC06/phys-v2-lumi2/00001820/DST/0000/" + datasetName + "_5.dst' TYP='POOL_ROOTTREE' OPT='READ'"]

elif inputType == "ETC":
    datasetName = "ETC_00002000_00000621"
    FileCatalog().Catalogs.append("xmlcatalog_file:$BRUNELROOT/job/" + datasetName + "_catalog.xml")
    # No 2008 ETC file yet, format is as follows:
    # EventSelector().Input = ["COLLECTION='TagCreator/1' DATAFILE='PFN:castor:/castor/cern.ch/user/p/pkoppenb/Stripping/" + datasetName + ".root'  TYP='POOL_ROOTTREE' SEL='(GlobalOr>=1)'"] # Generated by Joel from DC06 rDSTs
    
else:
    # Example POOL DIGI (default, MC production)
#    datasetName =  "11144101-100ev-20081006"
    # B->J/Psi(mumu)Ks events with Boole v16r3, Gauss v35r1, DB tag head-20081002
#    LHCbApp().DDDBtag   = "head-20081002"
#    LHCbApp().CondDBtag = "head-20081002"
#    EventSelector().Input = ["DATAFILE='PFN:castor:/castor/cern.ch/user/c/cattanem/Boole/v16r3/" + datasetName + ".digi' TYP='POOL_ROOTTREE' OPT='READ'"]

    datasetName =  "/Boole_450GeV_Boff_Beam12_CloseVelo_H_1000_15"
    # B->J/Psi(mumu)Ks events with Boole v17r2, Gauss v36r1, DB tag head-20090112
    #LHCbApp().DDDBtag   = "head-20090112"
    #LHCbApp().CondDBtag = "sim-20090112"
    LHCbApp().DDDBtag   = "default"
    LHCbApp().CondDBtag = "default"
    EventSelector().Input = ["DATAFILE='PFN:file:/data/user/data/Alignment/" + datasetName + ".digi' TYP='POOL_ROOTTREE' OPT='READ'"]

# /data/user/data/Alignment/Boole_450GeV_Boff_Beam12_CloseVelo_H_1000_15.digi
# Default output files names are set up using value Escher().DatasetName property
Escher().DatasetName = datasetName

# Redefine defaults by uncommenting one or more of options below 

# Monitoring histograms
#HistogramPersistencySvc().OutputFile = "SomeFile.root"

#-- Dst or rDst file
#OutputStream("DstWriter").Output = "DATAFILE='PFN:someFile.dst' TYP='POOL_ROOTTREE' OPT='REC'"
#OutputStream("DstWriter").Output = "DATAFILE='PFN:someFile.rdst' TYP='POOL_ROOTTREE' OPT='REC'"

#-- ETC output in case of ETC input
#TagCollectionSvc("EvtTupleSvc").Output = [ "EVTTAGS2 DATAFILE='some-etc.root' TYP='POOL_ROOTTREE' OPT='RECREATE' " ]
