# Example DC06 data files for Escher.
# This file must be given as last argument to gaudirun.py, after Escher<conf>.py

# Syntax is:
#   gaudirun.py Escher<conf>.py DC06-Files.py
#
from Gaudi.Configuration import *
from Configurables import Escher, LHCbApp

#-- File catalogs. First one is read-write
FileCatalog().Catalogs = [ "xmlcatalog_file:MyCatalog.xml",
                           "xmlcatalog_file:$BRUNELROOT/job/NewCatalog.xml" ]

# update the escher data type, just in case
Escher().DataType  = "DC06"

#-- Main ('signal') event input
inputType   = Escher().getProp("InputType").upper()


#-- DC06 database tags are backwards compatible by construction, use default
LHCbApp().DDDBtag   = "default"
LHCbApp().CondDBtag = "default"

if inputType == "MDF":
    # Example MDF file. File MUST be described in the FileCatalog, with an FID
    # generated externally, as it is not contained in the MDF file. If the FID is
    # not provided in the Catalog, the output dataset will not contain the
    # information to allow navigation back to the MDF data when reading
    datasetName =  "00001820_00000001"
    # B->J/Psi(mumu)Ks events with Boole v12r4, Gauss v25r2, XmlDDDB v30r8
    EventSelector().Input = [ "DATA='FID:94ACC0F5-09A3-DC11-8140-003048836626' SVC='LHCb::MDFSelector'" ] # Can use FID or LFN or PFN.

elif inputType == "DST":
    # Example POOL DST (reprocessing)
    datasetName =  "00001820_00000001"
    # B->J/Psi(mumu)Ks events with Boole v12r4, Gauss v25r2, XmlDDDB v30r8
    EventSelector().Input = ["DATAFILE='PFN:castor:/castor/cern.ch/grid/lhcb/production/DC06/phys-v2-lumi2/00001820/DST/0000/" + datasetName + "_5.dst' TYP='POOL_ROOTTREE' OPT='READ'"]

elif inputType == "ETC":
    # Example ETC
    datasetName = "ETC_00002000_00000621"
    FileCatalog().Catalogs.append("xmlcatalog_file:$BRUNELROOT/job/" + datasetName + "_catalog.xml")
    EventSelector().Input = ["COLLECTION='TagCreator/1' DATAFILE='PFN:castor:/castor/cern.ch/user/p/pkoppenb/Stripping/" + datasetName + ".root'  TYP='POOL_ROOTTREE' SEL='(GlobalOr>=1)'"] # Generated by Joel from DC06 rDSTs
    
else:
    # Example POOL DIGI (default, MC production)
    datasetName =  "00001820_00000001"
    # B->J/Psi(mumu)Ks events with Boole v12r4, Gauss v25r2, XmlDDDB v30r8
    files = ['00001820_00000001_4.digi','00001820_00000004_4.digi','00001820_00000009_4.digi','00001820_00000010_4.digi',
             '00001820_00000011_4.digi','00001820_00000014_4.digi','00001820_00000015_4.digi','00001820_00000016_4.digi',
             '00001820_00000018_4.digi','00001820_00000020_4.digi','00001820_00000024_4.digi','00001820_00000032_4.digi',
             '00001820_00000033_4.digi','00001820_00000040_4.digi']
    for file in files :
        filename = 'PFN:castor:/castor/cern.ch/grid/lhcb/production/DC06/phys-v2-lumi2/00001820/DIGI/0000/' + file
        EventSelector().Input.append("DATAFILE='" + filename + "' TYP='POOL_ROOTTREE' OPT='READ'")
        
#-- Example field off and velo open files
if "fieldOff" in Escher().getProp("SpecialData") :
    if "veloOpen" in Escher().getProp("SpecialData") :
        # 450 GeV on 450 GeV pp collisions, single interactions, no beam crossing angle, beam sigma(X) = sigma(Y) = 280 microns, magnetic field OFF, Velo Open
        datasetName = "B_OFF.VeloOpen"
        EventSelector().Input = ["DATAFILE='PFN:rfio:/castor/cern.ch/lhcb/background/Pilot_2007/30000000.5k." + datasetName + ".digi' TYP='POOL_ROOT' OPT='READ'"]
    else:
        # 450 GeV on 450 GeV pp collisions, single interactions, no beam crossing angle, beam sigma(X) = sigma(Y) = 280 microns, magnetic field OFF
        datasetName = "B_OFF.VeloClose"
        EventSelector().Input = ["DATAFILE='PFN:rfio:/castor/cern.ch/lhcb/background/Pilot_2007/30000000.5k." + datasetName + ".digi' TYP='POOL_ROOT' OPT='READ'"]

# Default output files names are set up using value Escher().DatasetName property
Escher().DatasetName = datasetName

# Redefine defaults by uncommenting one or more of options below 

# Monitoring histograms
#HistogramPersistencySvc().OutputFile = "SomeFile.root"

#-- Dst or rDst file
#OutputStream("DstWriter").Output = "DATAFILE='PFN:someFile.dst' TYP='POOL_ROOTTREE' OPT='REC'"
#OutputStream("DstWriter").Output = "DATAFILE='PFN:someFile.rdst' TYP='POOL_ROOTTREE' OPT='REC'"

#-- ETC output in case of ETC input
#TagCollectionSvc("EvtTupleSvc").Output = [ "EVTTAGS2 DATAFILE='some-etc.root' TYP='POOL_ROOTTREE' OPT='RECREATE' " ]
